// Copyright 2006 all rights reserved; see LICENSE file for BSD-style license

package edu.berkeley.sbp;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import java.lang.ref.*;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.util.*;

/** <font color=purple>a stream of <tt>Token</tt>s to be parsed</font> */
public interface Input<Token> {

    /** returns the token just beyond the current location and advances beyond it */
    public Token           next() throws IOException;

    /** returns the location the input stream is currently at */
    public Location<Token> getLocation();
    
    /** should return a short string describing where the input is coming from */
    public String getName();

    /**
     *  <b>Optional:</b> <i>If possible</i>, this method will return a
     *  <60 char long rendering of the input region (for example, if
     *  the input is a region of characters, it would be those
     *  characters, possibly with ellipses in the middle to truncate
     *  the length) -- otherwise, returns null.
     */
    public abstract String showRegion(Region<Token> r);

    /** <font color=purple>a location (position) in the input stream -- <i>between tokens</i></font> */
    public static interface Location<Token> extends Comparable<Location> {

        /** return the region between this location and <tt>loc</tt> */
        public Region<Token> createRegion(Location<Token> loc);

        public String toString();

        /** the location following this one */
        public Location next();

        /** the location preceding this one */
        public Location prev();
    }

    /** <font color=purple>a contiguous set of <tt>Location</tt>s</font> */
    public static interface Region<Token> /* implements Topology<Location<Tok>> */ {

        /**
         *  the toString() method of Region should return a <80char
         *  "rendition" of the input region, if possible
         */
        public abstract String toString();
        
        /** The location of the start of this region */
        public abstract Location<Token> getStart();

        /** The location of the end of this region */
        public abstract Location<Token> getEnd();

    }

}


